@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ class TelnetUrl (urlbase.UrlBase):
         label is "login: ", expected password label is "Password: ".
         """
         self.url_connection = telnetlib.Telnet()
-        if self.consumer.config("debug"):
+        if self.aggregate.config["debug"]:
             self.url_connection.set_debuglevel(1)
         self.url_connection.open(self.host, self.port)
         if self.user:
