@@ -24,11 +24,12 @@ import codecs
 import difflib
 import unittest
 
-import linkcheck
 import linkcheck.checker
 import linkcheck.configuration
 import linkcheck.logger
 
+# helper alias
+get_url_from = linkcheck.checker.get_url_from
 
 class TestLogger (linkcheck.logger.Logger):
     """
@@ -73,7 +74,10 @@ class TestLogger (linkcheck.logger.Logger):
             self.result.append(u"baseurl %s" % url_data.base_ref)
         if self.has_part('info'):
             for info in url_data.info:
-                self.result.append(u"info %s" % info[1])
+                line = info[1]
+                if "Last modified" not in line and \
+                   "is located in" not in line:
+                    self.result.append(u"info %s" % line)
         if self.has_part('warning'):
             for warning in url_data.warnings:
                 self.result.append(u"warning %s" % warning[1])
@@ -175,8 +179,7 @@ class LinkCheckTest (unittest.TestCase):
             confargs = {}
         logargs = {'expected': self.get_resultlines(filename)}
         aggregate = get_test_aggregate(confargs, logargs)
-        url_data = linkcheck.checker.get_url_from(
-                                url, 0, aggregate, assume_local=assume_local)
+        url_data = get_url_from(url, 0, aggregate, assume_local=assume_local)
         if assume_local:
             linkcheck.add_intern_pattern(url_data, aggregate.config)
         aggregate.urlqueue.put(url_data)
@@ -188,7 +191,7 @@ class LinkCheckTest (unittest.TestCase):
             l = sep.join(l)
             self.fail(l.encode("iso8859-1", "ignore"))
 
-    def direct (self, url, resultlines, fields=None, recursionlevel=0,
+    def direct (self, url, resultlines, parts=None, recursionlevel=0,
                 confargs=None, assume_local=False):
         """
         Check url with expected result.
@@ -199,11 +202,10 @@ class LinkCheckTest (unittest.TestCase):
         else:
             confargs['recursionlevel'] = recursionlevel
         logargs = {'expected': resultlines}
-        if fields is not None:
-            logargs['fields'] = fields
+        if parts is not None:
+            logargs['parts'] = parts
         aggregate = get_test_aggregate(confargs, logargs)
-        url_data = linkcheck.checker.get_url_from(
-                                url, 0, aggregate, assume_local=assume_local)
+        url_data = get_url_from(url, 0, aggregate, assume_local=assume_local)
         if assume_local:
             linkcheck.add_intern_pattern(url_data, aggregate.config)
         aggregate.urlqueue.put(url_data)
