@@ -20,11 +20,9 @@ Store metadata and options.
 
 import sys
 import os
-import logging
 import logging.config
 import urllib
 import _linkchecker_configdata
-import linkcheck
 import linkcheck.log
 import linkcheck.containers
 import confparse
